@@ -1,9 +1,14 @@
+from copy import deepcopy
 import logging
 import json
+import mock
 import treq
 from twisted.internet.defer import inlineCallbacks
 from twisted.web import http
 
+from treq.testing import StubTreq
+from treq.testing import RequestSequence, StringStubbingResource
+
 from vumi.message import TransportEvent, TransportUserMessage
 
 from junebug.channel import Channel
@@ -13,6 +18,9 @@ from junebug.utils import api_from_event, conjoin, omit
 
 
 class TestJunebugApi(JunebugTestBase):
+
+    maxDiff = None
+
     @inlineCallbacks
     def setUp(self):
         self.patch_logger()
@@ -22,6 +30,10 @@ class TestJunebugApi(JunebugTestBase):
         return treq.get(
             "%s%s" % (self.url, url), params=params, persistent=False)
 
+    def request(self, method, url, params={}):
+        return treq.request(
+            method, "%s%s" % (self.url, url), params=params, persistent=False)
+
     def post(self, url, data, headers=None):
         return treq.post(
             "%s%s" % (self.url, url),
@@ -29,6 +41,20 @@ class TestJunebugApi(JunebugTestBase):
             persistent=False,
             headers=headers)
 
+    def put(self, url, data, headers=None):
+        return treq.put(
+            "%s%s" % (self.url, url),
+            json.dumps(data),
+            persistent=False,
+            headers=headers)
+
+    def patch_request(self, url, data, headers=None):
+        return treq.patch(
+            "%s%s" % (self.url, url),
+            json.dumps(data),
+            persistent=False,
+            headers=headers)
+
     def delete(self, url):
         return treq.delete("%s%s" % (self.url, url), persistent=False)
 
@@ -56,7 +82,10 @@ class TestJunebugApi(JunebugTestBase):
             'the URL manually please check your spelling and try again.', {
                 'errors': [{
                     'code': 404,
-                    'message': '404: Not Found',
+                    'message': ('404 Not Found: The requested URL was not '
+                                'found on the server.  If you entered the URL'
+                                ' manually please check your spelling and try'
+                                ' again.'),
                     'type': 'Not Found',
                     }]
                 })
@@ -70,9 +99,9 @@ class TestJunebugApi(JunebugTestBase):
             None, {
                 'errors': [{
                     'code': 301,
-                    'message': '301: Moved Permanently',
-                    'type': 'Moved Permanently',
+                    'message': '301 Moved Permanently: None',
                     'new_url': '%s/channels/' % self.url,
+                    'type': 'Moved Permanently',
                 }],
             })
         yield self.assert_response(
@@ -174,7 +203,7 @@ class TestJunebugApi(JunebugTestBase):
         resp = yield self.post('/channels/', properties)
 
         yield self.assert_response(
-            resp, http.OK, 'channel created',
+            resp, http.CREATED, 'channel created',
             conjoin(properties, {'status': self.generate_status()}),
             ignore=['id'])
 
@@ -330,6 +359,46 @@ class TestJunebugApi(JunebugTestBase):
                 'id': 'test-channel',
             }))
 
+    @inlineCallbacks
+    def test_modify_channel_config_remove_mo_url(self):
+        redis = yield self.get_redis()
+        properties = self.create_channel_properties()
+        config = yield self.create_channel_config()
+
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        properties['config']['name'] = 'bar'
+        properties['mo_url'] = None
+        resp = yield self.post('/channels/test-channel', properties)
+
+        yield self.assert_response(
+            resp, http.OK, 'channel updated', conjoin(properties, {
+                'status': self.generate_status(),
+                'id': 'test-channel',
+            }))
+
+    @inlineCallbacks
+    def test_modify_channel_config_remove_status_url(self):
+        redis = yield self.get_redis()
+        properties = self.create_channel_properties()
+        config = yield self.create_channel_config()
+
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        properties['config']['name'] = 'bar'
+        properties['status_url'] = None
+        resp = yield self.post('/channels/test-channel', properties)
+
+        yield self.assert_response(
+            resp, http.OK, 'channel updated', conjoin(properties, {
+                'status': self.generate_status(),
+                'id': 'test-channel',
+            }))
+
     @inlineCallbacks
     def test_modify_channel_invalid_parameters(self):
         resp = yield self.post('/channels/foo-bar', {
@@ -457,10 +526,11 @@ class TestJunebugApi(JunebugTestBase):
         resp = yield self.post('/channels/test-channel/messages/', {
             'to': '+1234', 'content': 'foo', 'from': None})
         yield self.assert_response(
-            resp, http.OK, 'message sent', {
+            resp, http.CREATED, 'message submitted', {
                 'to': '+1234',
                 'channel_id': 'test-channel',
                 'from': None,
+                'group': None,
                 'reply_to': None,
                 'channel_data': {},
                 'content': 'foo',
@@ -474,6 +544,39 @@ class TestJunebugApi(JunebugTestBase):
             'test-channel', message['message_id'])
         self.assertEqual(event_url, None)
 
+    @inlineCallbacks
+    def test_send_group_message(self):
+        '''Sending a group message should place the message on the queue for the
+        channel'''
+        properties = self.create_channel_properties()
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+        resp = yield self.post('/channels/test-channel/messages/', {
+            'to': '+1234', 'content': 'foo', 'from': None,
+            'group': 'the-group'})
+        yield self.assert_response(
+            resp, http.CREATED, 'message submitted', {
+                'to': '+1234',
+                'channel_id': 'test-channel',
+                'from': None,
+                'group': 'the-group',
+                'reply_to': None,
+                'channel_data': {},
+                'content': 'foo',
+            }, ignore=['timestamp', 'message_id'])
+
+        [message] = self.get_dispatched_messages('test-channel.outbound')
+        message_id = (yield resp.json())['result']['message_id']
+        self.assertEqual(message['message_id'], message_id)
+        self.assertEqual(message['group'], 'the-group')
+
+        event_url = yield self.api.outbounds.load_event_url(
+            'test-channel', message['message_id'])
+        self.assertEqual(event_url, None)
+
     @inlineCallbacks
     def test_send_message_message_rate(self):
         '''Sending a message should increment the message rate counter'''
@@ -506,10 +609,11 @@ class TestJunebugApi(JunebugTestBase):
             'to': '+1234', 'content': 'foo', 'from': None,
             'event_url': 'http://test.org'})
         yield self.assert_response(
-            resp, http.OK, 'message sent', {
+            resp, http.CREATED, 'message submitted', {
                 'to': '+1234',
                 'channel_id': 'test-channel',
                 'from': None,
+                'group': None,
                 'reply_to': None,
                 'channel_data': {},
                 'content': 'foo',
@@ -519,6 +623,34 @@ class TestJunebugApi(JunebugTestBase):
             'test-channel', (yield resp.json())['result']['message_id'])
         self.assertEqual(event_url, 'http://test.org')
 
+    @inlineCallbacks
+    def test_send_message_event_auth_token(self):
+        '''Sending a message with a specified event url and auth token should
+        store the auth token for sending events in the future'''
+        properties = self.create_channel_properties()
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+        resp = yield self.post('/channels/test-channel/messages/', {
+            'to': '+1234', 'content': 'foo', 'from': None,
+            'event_url': 'http://test.org', 'event_auth_token': 'the_token'})
+        yield self.assert_response(
+            resp, http.CREATED, 'message submitted', {
+                'to': '+1234',
+                'channel_id': 'test-channel',
+                'from': None,
+                'group': None,
+                'reply_to': None,
+                'channel_data': {},
+                'content': 'foo',
+            }, ignore=['timestamp', 'message_id'])
+
+        event_auth_token = yield self.api.outbounds.load_event_auth_token(
+            'test-channel', (yield resp.json())['result']['message_id'])
+        self.assertEqual(event_auth_token, 'the_token')
+
     @inlineCallbacks
     def test_send_message_reply(self):
         '''Sending a reply message should fetch the relevant inbound message,
@@ -550,8 +682,8 @@ class TestJunebugApi(JunebugTestBase):
         })
 
         yield self.assert_response(
-            resp, http.OK,
-            'message sent',
+            resp, http.CREATED,
+            'message submitted',
             omit(expected, 'timestamp', 'message_id'),
             ignore=['timestamp', 'message_id'])
 
@@ -589,35 +721,78 @@ class TestJunebugApi(JunebugTestBase):
 
     @inlineCallbacks
     def test_send_message_both_to_and_reply_to(self):
-        resp = yield self.post('/channels/foo-bar/messages/', {
+
+        properties = self.create_channel_properties(character_limit=100)
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        resp = yield self.post('/channels/test-channel/messages/', {
             'from': None,
             'to': '+1234',
             'reply_to': '2e8u9ua8',
             'content': None,
         })
         yield self.assert_response(
-            resp, http.BAD_REQUEST, 'api usage error', {
+            resp, http.BAD_REQUEST, 'message not found', {
                 'errors': [{
-                    'message': 'Only one of "to" and "reply_to" may be '
-                    'specified',
-                    'type': 'ApiUsageError',
+                    'message': 'Inbound message with id 2e8u9ua8 not found',
+                    'type': 'MessageNotFound',
                 }]
             })
 
+    @inlineCallbacks
+    def test_send_message_both_to_and_reply_to_allowing_expiry(self):
+        properties = self.create_channel_properties(character_limit=100)
+        config = yield self.create_channel_config(
+            allow_expired_replies=True)
+        redis = yield self.get_redis()
+        yield self.stop_server()
+        yield self.start_server(config=config)
+
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        resp = yield self.post('/channels/test-channel/messages/', {
+            'from': None,
+            'to': '+1234',
+            'reply_to': '2e8u9ua8',
+            'content': 'foo',
+        })
+        yield self.assert_response(
+            resp, http.CREATED, 'message submitted', {
+                'channel_data': {},
+                'from': None,
+                'to': '+1234',
+                'content': 'foo',
+                'group': None,
+                'channel_id': u'test-channel',
+                'reply_to': None,
+            }, ignore=['timestamp', 'message_id'])
+
     @inlineCallbacks
     def test_send_message_from_and_reply_to(self):
-        resp = yield self.post('/channels/foo-bar/messages/', {
-            'from': '+1234',
+        properties = self.create_channel_properties(character_limit=100)
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        resp = yield self.post('/channels/test-channel/messages/', {
+            'from': None,
+            'to': '+1234',
             'reply_to': '2e8u9ua8',
             'content': None,
         })
-
         yield self.assert_response(
-            resp, http.BAD_REQUEST, 'api usage error', {
+            resp, http.BAD_REQUEST, 'message not found', {
                 'errors': [{
-                    'message': 'Only one of "from" and "reply_to" may be '
-                    'specified',
-                    'type': 'ApiUsageError',
+                    'message': 'Inbound message with id 2e8u9ua8 not found',
+                    'type': 'MessageNotFound',
                 }]
             })
 
@@ -635,10 +810,11 @@ class TestJunebugApi(JunebugTestBase):
             'to': '+1234', 'content': 'Under the character limit.',
             'from': None})
         yield self.assert_response(
-            resp, http.OK, 'message sent', {
+            resp, http.CREATED, 'message submitted', {
                 'to': '+1234',
                 'channel_id': 'test-channel',
                 'from': None,
+                'group': None,
                 'reply_to': None,
                 'channel_data': {},
                 'content': 'Under the character limit.',
@@ -659,10 +835,11 @@ class TestJunebugApi(JunebugTestBase):
         resp = yield self.post('/channels/test-channel/messages/', {
             'to': '+1234', 'content': content, 'from': None})
         yield self.assert_response(
-            resp, http.OK, 'message sent', {
+            resp, http.CREATED, 'message submitted', {
                 'to': '+1234',
                 'channel_id': 'test-channel',
                 'from': None,
+                'group': None,
                 'reply_to': None,
                 'channel_data': {},
                 'content': content,
@@ -754,6 +931,110 @@ class TestJunebugApi(JunebugTestBase):
         yield self.assert_response(
             resp, http.OK, 'health ok', {})
 
+    @inlineCallbacks
+    def test_get_channels_health_check(self):
+
+        config = yield self.create_channel_config(
+            rabbitmq_management_interface="rabbitmq:15672"
+        )
+        yield self.stop_server()
+        yield self.start_server(config=config)
+
+        channel = yield self.create_channel(self.service, self.redis)
+
+        request_list = []
+
+        for sub in ['inbound', 'outbound', 'event']:
+            queue_name = "%s.%s" % (channel.id, sub)
+            url = 'http://rabbitmq:15672/api/queues/%%2F/%s' % (queue_name)
+            request_list.append(
+                ((b'get', url, mock.ANY, mock.ANY, mock.ANY),
+                 (http.OK, {b'Content-Type': b'application/json'},
+                  b'{"messages": 1256, "messages_details": {"rate": 1.25}, "name": "%s"}' % queue_name)))  # noqa
+
+        async_failures = []
+        sequence_stubs = RequestSequence(request_list, async_failures.append)
+        stub_treq = StubTreq(StringStubbingResource(sequence_stubs))
+
+        def new_get(*args, **kwargs):
+            return stub_treq.request("GET", args[0])
+
+        with (mock.patch('treq.client.HTTPClient.get', side_effect=new_get)):
+            with sequence_stubs.consume(self.fail):
+                resp = yield self.request('GET', '/health')
+
+            yield self.assertEqual(async_failures, [])
+            yield self.assert_response(
+                resp, http.OK, 'queues ok', [
+                    {
+                        'messages': 1256,
+                        'name': '%s.inbound' % (channel.id),
+                        'rate': 1.25,
+                        'stuck': False
+                    }, {
+                        'messages': 1256,
+                        'name': '%s.outbound' % (channel.id),
+                        'rate': 1.25,
+                        'stuck': False
+                    }, {
+                        'messages': 1256,
+                        'name': '%s.event' % (channel.id),
+                        'rate': 1.25,
+                        'stuck': False
+                    }])
+
+    @inlineCallbacks
+    def test_get_channels_health_check_stuck(self):
+
+        config = yield self.create_channel_config(
+            rabbitmq_management_interface="rabbitmq:15672"
+        )
+        yield self.stop_server()
+        yield self.start_server(config=config)
+
+        channel = yield self.create_channel(self.service, self.redis)
+
+        request_list = []
+
+        for sub in ['inbound', 'outbound', 'event']:
+            queue_name = "%s.%s" % (channel.id, sub)
+            url = 'http://rabbitmq:15672/api/queues/%%2F/%s' % (queue_name)
+            request_list.append(
+                ((b'get', url, mock.ANY, mock.ANY, mock.ANY),
+                 (http.OK, {b'Content-Type': b'application/json'},
+                  b'{"messages": 1256, "messages_details": {"rate": 0}, "name": "%s"}' % queue_name)))  # noqa
+
+        async_failures = []
+        sequence_stubs = RequestSequence(request_list, async_failures.append)
+        stub_treq = StubTreq(StringStubbingResource(sequence_stubs))
+
+        def new_get(*args, **kwargs):
+            return stub_treq.request("GET", args[0])
+
+        with (mock.patch('treq.client.HTTPClient.get', side_effect=new_get)):
+            with sequence_stubs.consume(self.fail):
+                resp = yield self.request('GET', '/health')
+
+            yield self.assertEqual(async_failures, [])
+            yield self.assert_response(
+                resp, http.INTERNAL_SERVER_ERROR, 'queues stuck', [
+                    {
+                        'messages': 1256,
+                        'name': '%s.inbound' % (channel.id),
+                        'rate': 0,
+                        'stuck': True
+                    }, {
+                        'messages': 1256,
+                        'name': '%s.outbound' % (channel.id),
+                        'rate': 0,
+                        'stuck': True
+                    }, {
+                        'messages': 1256,
+                        'name': '%s.event' % (channel.id),
+                        'rate': 0,
+                        'stuck': True
+                    }])
+
     @inlineCallbacks
     def test_get_channel_logs_no_logs(self):
         '''If there are no logs, an empty list should be returned.'''
@@ -896,3 +1177,240 @@ class TestJunebugApi(JunebugTestBase):
             'logger': channel.id,
             'message': 'Test2',
             'level': logging.INFO})
+
+    @inlineCallbacks
+    def test_get_router_list(self):
+        '''A GET request on the routers collection endpoint should result in
+        the list of router UUIDs being returned'''
+        redis = yield self.get_redis()
+
+        resp = yield self.get('/routers/')
+        yield self.assert_response(resp, http.OK, 'routers retrieved', [])
+
+        yield redis.sadd('routers', '64f78582-8e83-40c9-be23-cc93d54e9dcd')
+
+        resp = yield self.get('/routers/')
+        yield self.assert_response(resp, http.OK, 'routers retrieved', [
+            u'64f78582-8e83-40c9-be23-cc93d54e9dcd',
+        ])
+
+        yield redis.sadd('routers', 'ceee6a83-fa6b-42d2-b65f-1a1cf85ac6f8')
+
+        resp = yield self.get('/routers/')
+        yield self.assert_response(resp, http.OK, 'routers retrieved', [
+            u'64f78582-8e83-40c9-be23-cc93d54e9dcd',
+            u'ceee6a83-fa6b-42d2-b65f-1a1cf85ac6f8',
+        ])
+
+    @inlineCallbacks
+    def test_create_router(self):
+        """Creating a router with a valid config should succeed"""
+        config = self.create_router_config()
+        resp = yield self.post('/routers/', config)
+
+        yield self.assert_response(
+            resp, http.CREATED, 'router created', config, ignore=['id'])
+
+    @inlineCallbacks
+    def test_create_router_invalid_worker_config(self):
+        """The worker config should be sent to the router for validation"""
+        config = self.create_router_config(config={'test': 'fail'})
+        resp = yield self.post('/routers/', config)
+
+        yield self.assert_response(
+            resp, http.BAD_REQUEST, 'invalid router config', {
+                'errors': [{
+                    'message': 'test must be pass',
+                    'type': 'InvalidRouterConfig',
+                }]
+            })
+
+    @inlineCallbacks
+    def test_create_router_worker(self):
+        """When creating a new router, the router worker should successfully
+        be started"""
+        config = self.create_router_config()
+        resp = yield self.post('/routers/', config)
+
+        # Check that the worker is created with the correct config
+        id = (yield resp.json())['result']['id']
+        transport = self.service.namedServices[id]
+
+        self.assertEqual(transport.parent, self.service)
+
+        self.assertEqual(transport.config, config['config'])
+
+    @inlineCallbacks
+    def test_create_router_saves_config(self):
+        """When creating a worker, the config should be saved inside the router
+        store"""
+        config = self.create_router_config()
+        resp = yield self.post('/routers/', config)
+
+        routers = yield self.api.router_store.get_router_list()
+        self.assertEqual(routers, [(yield resp.json())['result']['id']])
+
+    @inlineCallbacks
+    def test_get_router(self):
+        """The get router endpoint should return the config and status of the
+        specified router"""
+        config = self.create_router_config()
+        resp = yield self.post('/routers/', config)
+
+        router_id = (yield resp.json())['result']['id']
+        resp = yield self.get('/routers/{}'.format(router_id))
+        self.assert_response(
+            resp, http.OK, 'router found', config, ignore=['id'])
+
+    @inlineCallbacks
+    def test_get_non_existing_router(self):
+        """If a router for the given ID does not exist, then a not found error
+        should be returned"""
+        resp = yield self.get('/routers/bad-router-id')
+        self.assert_response(resp, http.NOT_FOUND, 'router not found', {
+            'errors': [{
+                'message': 'Router with ID bad-router-id cannot be found',
+                'type': 'RouterNotFound',
+            }]
+        })
+
+    @inlineCallbacks
+    def test_replace_router_config(self):
+        """When creating a PUT request, the router configuration should be
+        replaced"""
+        old_config = self.create_router_config(label='test', config={
+            'test': 'pass', 'foo': 'bar'})
+        resp = yield self.post('/routers/', old_config)
+        router_id = (yield resp.json())['result']['id']
+
+        router_config = yield self.api.router_store.get_router_config(
+            router_id)
+        old_config['id'] = router_id
+        self.assertEqual(router_config, old_config)
+        router_worker = self.api.service.namedServices[router_id]
+        self.assertEqual(router_worker.config, old_config['config'])
+
+        new_config = self.create_router_config(config={'test': 'pass'})
+        new_config.pop('label', None)
+        resp = yield self.put('/routers/{}'.format(router_id), new_config)
+        new_config['id'] = router_id
+
+        yield self.assert_response(
+            resp, http.OK, 'router updated', new_config)
+
+        router_config = yield self.api.router_store.get_router_config(
+            router_id)
+        self.assertEqual(router_config, new_config)
+        router_worker = self.api.service.namedServices[router_id]
+        self.assertEqual(router_worker.config, new_config['config'])
+
+        router_worker = self.api.service.namedServices[router_id]
+
+    @inlineCallbacks
+    def test_replace_router_config_invalid_worker_config(self):
+        """Before replacing the worker config, the new config should be
+        validated."""
+        old_config = self.create_router_config(config={'test': 'pass'})
+        resp = yield self.post('/routers/', old_config)
+        router_id = (yield resp.json())['result']['id']
+
+        new_config = self.create_router_config(config={'test': 'fail'})
+        resp = yield self.put('/routers/{}'.format(router_id), new_config)
+
+        yield self.assert_response(
+            resp, http.BAD_REQUEST, 'invalid router config', {
+                'errors': [{
+                    'message': 'test must be pass',
+                    'type': 'InvalidRouterConfig',
+                }]
+            })
+
+        resp = yield self.get('/routers/{}'.format(router_id))
+        yield self.assert_response(
+            resp, http.OK, 'router found', old_config, ignore=['id'])
+
+    @inlineCallbacks
+    def test_update_router_config(self):
+        """When creating a PATCH request, the router configuration should be
+        updated"""
+        old_config = self.create_router_config(
+            label='old', config={'test': 'pass'})
+        resp = yield self.post('/routers/', old_config)
+        router_id = (yield resp.json())['result']['id']
+
+        router_config = yield self.api.router_store.get_router_config(
+            router_id)
+        old_config['id'] = router_id
+        self.assertEqual(router_config, old_config)
+        router_worker = self.api.service.namedServices[router_id]
+        self.assertEqual(router_worker.config, old_config['config'])
+
+        update = {'config': {'test': 'pass', 'new': 'new'}}
+        new_config = deepcopy(old_config)
+        new_config.update(update)
+        self.assertEqual(new_config['label'], 'old')
+        resp = yield self.patch_request(
+            '/routers/{}'.format(router_id), update)
+
+        yield self.assert_response(
+            resp, http.OK, 'router updated', new_config)
+
+        router_config = yield self.api.router_store.get_router_config(
+            router_id)
+        self.assertEqual(router_config, new_config)
+        router_worker = self.api.service.namedServices[router_id]
+        self.assertEqual(router_worker.config, new_config['config'])
+
+        router_worker = self.api.service.namedServices[router_id]
+
+    @inlineCallbacks
+    def test_update_router_config_invalid_worker_config(self):
+        """Before updating the worker config, the new config should be
+        validated."""
+        old_config = self.create_router_config(config={'test': 'pass'})
+        resp = yield self.post('/routers/', old_config)
+        router_id = (yield resp.json())['result']['id']
+
+        update = {'config': {'test': 'fail'}}
+        resp = yield self.patch_request(
+            '/routers/{}'.format(router_id), update)
+
+        yield self.assert_response(
+            resp, http.BAD_REQUEST, 'invalid router config', {
+                'errors': [{
+                    'message': 'test must be pass',
+                    'type': 'InvalidRouterConfig',
+                }]
+            })
+
+        resp = yield self.get('/routers/{}'.format(router_id))
+        yield self.assert_response(
+            resp, http.OK, 'router found', old_config, ignore=['id'])
+
+    @inlineCallbacks
+    def test_delete_router(self):
+        """Should stop the router from running, and delete its config"""
+        config = self.create_router_config()
+        resp = yield self.post('/routers/', config)
+        router_id = (yield resp.json())['result']['id']
+
+        self.assertTrue(router_id in self.service.namedServices)
+        routers = yield self.api.router_store.get_router_list()
+        self.assertEqual(routers, [router_id])
+
+        resp = yield self.delete('/routers/{}'.format(router_id))
+        self.assert_response(resp, http.OK, 'router deleted', {})
+        self.assertFalse(router_id in self.service.namedServices)
+        routers = yield self.api.router_store.get_router_list()
+        self.assertEqual(routers, [])
+
+    @inlineCallbacks
+    def test_delete_non_existing_router(self):
+        """Should return a router not found"""
+        resp = yield self.delete('/routers/bad-id')
+        self.assert_response(resp, http.NOT_FOUND, 'router not found', {
+            'errors': [{
+                'message': 'Router with ID bad-id cannot be found',
+                'type': 'RouterNotFound',
+            }]
+        })
