@@ -1,9 +1,10 @@
+import json
 from twisted.internet.defer import inlineCallbacks, returnValue
 from vumi.message import TransportEvent, TransportUserMessage, TransportStatus
 
 from junebug.stores import (
     BaseStore, InboundMessageStore, OutboundMessageStore, StatusStore,
-    MessageRateStore)
+    MessageRateStore, RouterStore)
 from junebug.tests.helpers import JunebugTestBase
 
 
@@ -86,6 +87,26 @@ class TestBaseStore(JunebugTestBase):
 
         self.assertEqual(val, None)
 
+    @inlineCallbacks
+    def test_remove_property(self):
+        '''remove_property should remove the specified property from the
+        specified id'''
+        store = yield self.create_store()
+
+        yield self.redis.hmset('testid', {
+            'prop1': 'foo',
+            'prop2': 'bar',
+        })
+        self.assertEqual((yield self.redis.hgetall('testid')), {
+            'prop1': 'foo',
+            'prop2': 'bar',
+        })
+
+        yield store.remove_property('testid', 'prop1')
+        self.assertEqual((yield self.redis.hgetall('testid')), {
+            'prop2': 'bar',
+        })
+
     @inlineCallbacks
     def test_override_ttl(self):
         '''If a ttl for an action is specified, it should override the default
@@ -134,6 +155,73 @@ class TestBaseStore(JunebugTestBase):
         yield store.get_id('testid3', ttl=None)
         self.assertEqual((yield self.redis.ttl('testid3')), None)
 
+    @inlineCallbacks
+    def test_get_set(self):
+        '''get_set returns the set of values stored at the specified id'''
+        store = yield self.create_store()
+
+        self.assertEqual((yield store.get_set('testid')), set())
+
+        yield self.redis.sadd('testid', 'item1')
+        self.assertEqual((yield store.get_set('testid')), set(('item1',)))
+
+        yield self.redis.sadd('testid', 'item2')
+        self.assertEqual(
+            (yield store.get_set('testid')),
+            set(('item1', 'item2'))
+        )
+
+    @inlineCallbacks
+    def test_remove_set_item(self):
+        '''remove_set_item removes a specific item from the set stored at the
+        specified id'''
+        store = yield self.create_store()
+
+        yield self.redis.sadd('testid', 'item1')
+        yield self.redis.sadd('testid', 'item2')
+        self.assertEqual(
+            (yield self.redis.smembers('testid')), set(('item1', 'item2')))
+
+        yield store.remove_set_item('testid', 'item1')
+        self.assertEqual(
+            (yield self.redis.smembers('testid')), set(('item2',)))
+
+    @inlineCallbacks
+    def test_store_value(self):
+        '''stores the given value at the given key'''
+        store = yield self.create_store()
+
+        self.assertEqual((yield self.redis.get('testid')), None)
+
+        yield store.store_value('testid', 'testvalue')
+
+        self.assertEqual((yield self.redis.get('testid')), 'testvalue')
+
+    @inlineCallbacks
+    def test_load_value(self):
+        '''loads the value stored at the given key'''
+        store = yield self.create_store()
+
+        self.assertEqual((yield store.load_value('testid')), None)
+
+        yield self.redis.set('testid', 'testvalue')
+
+        self.assertEqual((yield store.load_value('testid')), 'testvalue')
+
+    @inlineCallbacks
+    def test_remove_value(self):
+        '''removes the value stored at the given key'''
+        store = yield self.create_store()
+
+        yield store.remove_value('testid')
+        self.assertEqual((yield self.redis.get('testid')), None)
+
+        yield self.redis.set('testid', 'testvalue')
+        self.assertEqual((yield self.redis.get('testid')), 'testvalue')
+
+        yield store.remove_value('testid')
+        self.assertEqual((yield self.redis.get('testid')), None)
+
 
 class TestInboundMessageStore(JunebugTestBase):
     @inlineCallbacks
@@ -191,6 +279,16 @@ class TestOutboundMessageStore(JunebugTestBase):
             'channel_id:outbound_messages:messageid', 'event_url')
         self.assertEqual(event_url, 'http://test.org')
 
+    @inlineCallbacks
+    def test_store_event_auth_token(self):
+        '''Stores the event auth token under the message ID'''
+        store = yield self.create_store()
+        yield store.store_event_auth_token(
+            'channel_id', 'messageid', "the-auth-token")
+        event_auth_token = yield self.redis.hget(
+            'channel_id:outbound_messages:messageid', 'event_url_auth_token')
+        self.assertEqual(event_auth_token, "the-auth-token")
+
     @inlineCallbacks
     def test_load_event_url(self):
         '''Returns a vumi message from the stored json'''
@@ -204,6 +302,19 @@ class TestOutboundMessageStore(JunebugTestBase):
             'channel_id', vumi_msg.get('message_id'))
         self.assertEqual(event_url, 'http://test.org')
 
+    @inlineCallbacks
+    def test_load_event_auth_token(self):
+        '''Returns the event auth token under the message ID'''
+        store = yield self.create_store()
+        vumi_msg = TransportUserMessage.send(to_addr='+213', content='foo')
+        yield self.redis.hset(
+            'channel_id:outbound_messages:%s' % vumi_msg.get('message_id'),
+            'event_url_auth_token', "the-auth-token")
+
+        event_auth_token = yield store.load_event_auth_token(
+            'channel_id', vumi_msg.get('message_id'))
+        self.assertEqual(event_auth_token, "the-auth-token")
+
     @inlineCallbacks
     def test_load_event_url_not_exist(self):
         '''`None` should be returned if the message cannot be found'''
@@ -211,6 +322,13 @@ class TestOutboundMessageStore(JunebugTestBase):
         self.assertEqual((yield store.load_event_url(
             'bad-channel', 'bad-id')), None)
 
+    @inlineCallbacks
+    def test_load_event_auth_token_not_exist(self):
+        '''`None` should be returned if the event auth token cannot be found'''
+        store = yield self.create_store()
+        self.assertEqual((yield store.load_event_auth_token(
+            'bad-channel', 'bad-id')), None)
+
     @inlineCallbacks
     def test_store_event(self):
         '''Stores the event under the message ID'''
@@ -464,3 +582,83 @@ class TestMessageRateStore(JunebugTestBase):
         self.assertEqual((yield self.redis.get(bucket0)), None)
         self.assertEqual((yield self.redis.get(bucket1)), '1')
         self.assertEqual((yield self.redis.get(bucket2)), '1')
+
+
+class TestRouterStore(JunebugTestBase):
+    @inlineCallbacks
+    def create_store(self):
+        redis = yield self.get_redis()
+        store = RouterStore(redis)
+        returnValue(store)
+
+    @inlineCallbacks
+    def test_get_router_list(self):
+        store = yield self.create_store()
+
+        self.assertEqual((yield store.get_router_list()), [])
+
+        yield self.redis.sadd(
+            'routers', '64f78582-8e83-40c9-be23-cc93d54e9dcd')
+        self.assertEqual(
+            (yield store.get_router_list()),
+            ['64f78582-8e83-40c9-be23-cc93d54e9dcd'])
+
+        yield self.redis.sadd(
+            'routers', 'ceee6a83-fa6b-42d2-b65f-1a1cf85ac6f8')
+        self.assertEqual(
+            (yield store.get_router_list()),
+            ['64f78582-8e83-40c9-be23-cc93d54e9dcd',
+             'ceee6a83-fa6b-42d2-b65f-1a1cf85ac6f8'])
+
+    @inlineCallbacks
+    def test_save_router(self):
+        """save_router should save the router config at the router's uuid and
+        add the uuid to the router list"""
+        store = yield self.create_store()
+
+        config = self.create_router_config(id='test-uuid')
+        yield store.save_router(config)
+        self.assertEqual(
+            (yield self.redis.smembers('routers')), set(['test-uuid']))
+        self.assertEqual(
+            (yield self.redis.get('routers:test-uuid')), json.dumps(config))
+
+    @inlineCallbacks
+    def test_get_router_config(self):
+        """get_router_config should return the router config for the uuid
+        specified"""
+        store = yield self.create_store()
+
+        config = self.create_router_config(id='test-uuid')
+        yield self.redis.set('routers:test-uuid', json.dumps(config))
+
+        value = yield store.get_router_config('test-uuid')
+        self.assertEqual(value, config)
+
+    @inlineCallbacks
+    def test_get_router_config_doesnt_exist(self):
+        """If we don't have a config stored for the specified router ID, then
+        we should return None"""
+        store = yield self.create_store()
+        value = yield store.get_router_config('bad-uuid')
+        self.assertEqual(value, None)
+
+    @inlineCallbacks
+    def test_delete_router(self):
+        """Removes router's config and removes ID from router list"""
+        store = yield self.create_store()
+
+        config = self.create_router_config(id='test-uuid')
+        yield self.redis.sadd('routers', 'test-uuid')
+        yield self.redis.set('routers:test-uuid', json.dumps(config))
+        self.assertEqual(
+            (yield self.redis.smembers('routers')), set(['test-uuid']))
+        self.assertEqual(
+            (yield self.redis.get('routers:test-uuid')), json.dumps(config))
+
+        yield store.delete_router('test-uuid')
+
+        self.assertEqual(
+            (yield self.redis.smembers('routers')), set())
+        self.assertEqual(
+            (yield self.redis.get('routers:test-uuid')), None)
