@@ -1,4 +1,5 @@
 import treq
+from base64 import b64encode
 
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks, returnValue
@@ -91,6 +92,46 @@ class TestMessageForwardingWorker(JunebugTestBase):
 
         self.assertEqual(dispatched_msg, msg)
 
+    @inlineCallbacks
+    def test_send_message_with_basic_auth(self):
+        '''If there is an error sending a message to the configured URL, the
+        error and message should be logged'''
+        self.patch_logger()
+
+        # Inject the auth parameters
+        url = self.url.replace('http://', 'http://foo:bar@') + '/auth/'
+
+        self.worker = yield self.get_worker({
+            'transport_name': 'testtransport',
+            'mo_message_url': url,
+        })
+        msg = TransportUserMessage.send(to_addr='+1234', content='testcontent')
+        yield self.worker.consume_user_message(msg)
+
+        [logged_request] = self.logging_api.requests
+        self.assertEqual(logged_request, {
+            'Authorization': ['Basic %s' % (b64encode('foo:bar'),)]
+        })
+
+    @inlineCallbacks
+    def test_send_message_with_token_auth(self):
+        '''If there is an error sending a message to the configured URL, the
+        error and message should be logged'''
+        self.patch_logger()
+
+        self.worker = yield self.get_worker({
+            'transport_name': 'testtransport',
+            'mo_message_url': self.url + '/auth/',
+            'mo_message_url_auth_token': 'the-auth-token'
+        })
+        msg = TransportUserMessage.send(to_addr='+1234', content='testcontent')
+        yield self.worker.consume_user_message(msg)
+
+        [logged_request] = self.logging_api.requests
+        self.assertEqual(logged_request, {
+            'Authorization': ['Token the-auth-token']
+        })
+
     @inlineCallbacks
     def test_send_message_bad_response(self):
         '''If there is an error sending a message to the configured URL, the
@@ -108,6 +149,22 @@ class TestMessageForwardingWorker(JunebugTestBase):
         self.assert_was_logged('500')
         self.assert_was_logged('test-error-response')
 
+    @inlineCallbacks
+    def test_send_message_imploding_response(self):
+        '''If there is an error connecting to the configured URL, the
+        error and message should be logged'''
+        self.patch_logger()
+        self.worker = yield self.get_worker({
+            'transport_name': 'testtransport',
+            'mo_message_url': self.url + '/implode/',
+            })
+        msg = TransportUserMessage.send(to_addr='+1234', content='testcontent')
+        yield self.worker.consume_user_message(msg)
+
+        self.assert_was_logged('Post to %s/implode/ failed because of' % (
+            self.url,))
+        self.assert_was_logged('ConnectionDone')
+
     @inlineCallbacks
     def test_send_message_storing(self):
         '''Inbound messages should be stored in the InboundMessageStore'''
@@ -175,6 +232,49 @@ class TestMessageForwardingWorker(JunebugTestBase):
             body=api_from_event(self.worker.channel_id, event))
         yield self.assert_event_stored(event)
 
+    @inlineCallbacks
+    def test_forward_ack_http_with_token_auth(self):
+        event = TransportEvent(
+            event_type='ack',
+            user_message_id='msg-21',
+            sent_message_id='msg-21',
+            timestamp='2015-09-22 15:39:44.827794')
+
+        yield self.worker.outbounds.store_event_url(
+            self.worker.channel_id, 'msg-21', self.url + '/auth/')
+        yield self.worker.outbounds.store_event_auth_token(
+            self.worker.channel_id, 'msg-21', "the-auth-token"
+        )
+
+        yield self.worker.consume_ack(event)
+        [req] = self.logging_api.requests
+
+        self.assertEqual(req, {
+            'Authorization': ['Token the-auth-token']
+        })
+        yield self.assert_event_stored(event)
+
+    @inlineCallbacks
+    def test_forward_ack_http_with_basic_auth(self):
+        event = TransportEvent(
+            event_type='ack',
+            user_message_id='msg-21',
+            sent_message_id='msg-21',
+            timestamp='2015-09-22 15:39:44.827794')
+
+        # Inject the auth parameters
+        url = self.url.replace('http://', 'http://foo:bar@') + '/auth/'
+        yield self.worker.outbounds.store_event_url(
+            self.worker.channel_id, 'msg-21', url)
+
+        yield self.worker.consume_ack(event)
+        [req] = self.logging_api.requests
+
+        self.assertEqual(req, {
+            'Authorization': ['Basic %s' % (b64encode('foo:bar'),)]
+        })
+        yield self.assert_event_stored(event)
+
     @inlineCallbacks
     def test_forward_ack_amqp(self):
         '''A sent ack event should be forwarded to the correct AMQP queue if
